import { Query } from '@nestjs/graphql';
import {
  Args,
  Parent,
  ResolveField,
  Resolver,
  Subscription,
} from '@nestjs/graphql';
import { PipelineTasksService } from '../pipeline-tasks/pipeline-tasks.service';
import { Commit, LogFields, LogList } from '../repos/dtos/log-list.model';
import { ReposService } from '../repos/repos.service';
import { PipelinesService } from './pipelines.service';

@Resolver(() => Commit)
export class CommitLogsResolver {
  constructor(
    private readonly service: PipelinesService,
    private readonly taskServices: PipelineTasksService,
    private readonly reposService: ReposService,
  ) {}
  @Subscription(() => LogList, {
    resolve: (value) => {
      return value;
    },
  })
  async listLogsForPipeline(@Args('id', { type: () => String }) id: string) {
    const job = await this.service.listLogsForPipeline(id);
    return (async function* () {
      yield await job.finished();
    })();
  }
  @ResolveField()
  async tasks(@Parent() commit: LogFields) {
    return await this.taskServices.listTasksByCommitHash(commit.hash);
  }

  @Query(() => [Commit])
  async commits(@Args('pipelineId', { type: () => String }) id: string) {
    const pipeline = await this.service.findOneWithProject(id);
    const data = await this.reposService.listCommits(
      pipeline.project,
      pipeline.branch,
    );
    return data;
  }
}
